#!/usr/bin/env python3
"""Evaluates the results of our mini survey and throws them into some numbers
we can present."""

import csv

import numpy

## Set matplotlib output defaults
from matplotlib import rc
rc('font',**{'family':'serif','serif':['Times']}) # Times is IEEE default afaik
rc('text', usetex=True) # As advised by https://matplotlib.org/users/usetex.html

def load_result(filename):
    """Opens the filename and parses the results from it."""
    results = {}
    with open(filename) as file:
        reader = csv.reader(file, delimiter=',')
        next(reader, 0)
        for row in reader:
            if row[1] != '':
                sense = parse_sense(row[2])
                improve = parse_improvement(row[3])
                results[row[0]] = {
                    'sense': sense,
                    'improve': improve,
                    'category': row[1]
                    }
    return results

def parse_sense(text):
    """Given the text answer to the question whether our tool's output makes
    sense, return the number associated with it.  Numbers range from 1 to 5 for
    the 5 different answers that can be given. Follows Likert scale idea."""
    options = [
        'not at all',
        'not really',
        'neutral',
        'somewhat',
        'completely'
        ]
    return options.index(text) + 1

def parse_improvement(text):
    """Given the text answer to the question how to improve our tool's output,
    return the number associated with it.  Numbers range from 1 to 3. 1 = split
    clusters, 2 = leave as is, 3 = combine clusters."""
    options = [
        'split it further',
        'leave it as is',
        'combine some'
        ]
    return options.index(text) + 1

def clean_results(results):
    """Destructive on results. For each of its elements, checks their keys. If
    the keys are not present in all other elements, then the keys are deleted
    everywhere."""
    # Gather keys to delete
    todelete = set()
    for result in results:
        for key in result:
            for other in results:
                if not key in other:
                    todelete.add(key)
    # Delete keys
    for key in todelete:
        for result in results:
            try:
                del result[key]
            except KeyError:
                pass

def sense_vis(results):
    import numpy as np
    import matplotlib.pyplot as plt
    #plt.style.use('seaborn-deep')
    plt.style.use('seaborn-colorblind')

    person1 = results[0]
    person1 = list(map(lambda commit: person1[commit]['sense'], person1))
    person2 = results[1]
    person2 = list(map(lambda commit: person2[commit]['sense'], person2))
    person3 = results[2]
    person3 = list(map(lambda commit: person3[commit]['sense'], person3))

    bins = [0.5, 1.5, 2.5, 3.5, 4.5, 5.5]

    plt.hist([person1, person2, person3], bins, label=['Person 1', 'Person 2', 'Person 3'])
    plt.legend(loc='upper left')
    plt.xticks(np.arange(6), ('', 'Not at all', 'Not really', 'Neutral', 'Somewhat', 'Completely'))
    plt.ylabel('Number of replies')
    import time
    plt.savefig(str(int(round(time.time() * 1000))) + '.pdf', bbox_inches='tight')
    #plt.show()

def aggregate_sense(results):
    """Given a parsed results object, tries to come up with a meaningful number describing
    the answers given to the "sense" question."""
    all_senses = []
    for key in results[0]:
        senses = list(map(lambda x: x[key]['sense'], results))
        avg = sum(senses) / 3
        all_senses.append(avg)
    return numpy.average(all_senses)

def sense_per_person(results):
    senses = []
    for result in results:
        sense = list(map(lambda commit: result[commit]['sense'], result))
        avg = numpy.average(sense)
        senses.append(avg)
    return senses

def analyse_improvement(results):
    """Summarise the improvement possibilities"""
    split = 0
    combine = 0
    for key in results[0]:
        improvements = list(filter(lambda i: i != 2, map(lambda x: x[key]['improve'], results)))
        if not improvements:
            continue
        if max(improvements) == 3:
            combine = combine + 1
        else:
            split = split + 1
    return (split, combine)

if __name__ == '__main__':
    results = [
        load_result('./cluster-validity-person1.csv'),
        load_result('./cluster-validity-person2.csv'),
        load_result('./cluster-validity-person3.csv')
        ]
    clean_results(results)
    print('# of commits', len(results[0]))
    print('aggregated sense', aggregate_sense(results))
    sense_pp = sense_per_person(results)
    print('sense per person', sense_pp)
    print('sense per person avg', numpy.average(sense_pp))
    improv = analyse_improvement(results)
    print('improvement split', improv[0])
    print('improvement combine', improv[1])
    sense_vis(results)
