#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import csv

class Result:
    """Represents one result of the analysis done in Java."""
    def __init__(self, csvrow):
        self.sha = csvrow[0]
        self.file = csvrow[1]
        self.total_changes = int(csvrow[2])
        self.comment_changes = int(csvrow[3])
        self.no_original_changse = int(csvrow[4])
        self.no_method_changes = int(csvrow[5])
        if csvrow[6] == 'true':
            self.errored = True
        else:
            self.errored = False
        self.no_node_found = int(csvrow[7])
        self.node_found = int(csvrow[8])
        self.clusters = int(csvrow[9])

    def __str__(self):
        return "{sha},{file}: {found} ({total}) changes, {clusters} clusters" \
                .format(sha=self.sha, file=self.file, found=self.node_found, total=self.total_changes, clusters=self.clusters)

    def __repr__(self):
        return self.__str__()

def load_results(project_names, categories, result_folder):
    """Opens the files from the Java analysis, builds results object, returns
    it."""
    results = {}
    for project in project_names:
        if not project in results:
            results[project] = {}
        for category in categories:
            if not category in results[project]:
                results[project][category] = []
            path = result_folder + project + '.' + category + '.csv'
            with open(path) as file:
                reader = csv.reader(file, delimiter=',')
                for row in reader:
                    results[project][category].append(Result(row))
    return results


def filter_results(results, project_names, categories):
    """Creates a new results object with removal of files that errored or have
    no SDG nodes found."""
    new_results = {}
    for project in project_names:
        new_results[project] = {}
        for category in categories:
            new_results[project][category] = []
            for result in results[project][category]:
                if result.errored:
                    # Remember this also covers all non-java files since we
                    # throw an error on those.
                    continue
                if result.node_found == 0:
                    continue
                new_results[project][category].append(result)
    return new_results

def load_blob_info(project_names, folderpath):
    """Reads the dataset's obvious blobs files and parses how many partitions
    there should be if the information is present."""
    results = {}
    for project in project_names:
        results[project] = {}
        path = folderpath + project + '_obvious_blobs.csv'
        with open(path) as file:
            reader = csv.reader(file, delimiter=',')
            for row in reader:
                try:
                    results[project][row[0]] = int(row[1])
                except ValueError:
                    continue
    return results


def parse_bool(s, default_value=False):
    """Try to parse the input to a bool or return the default value
    otherwise."""
    try:
        return bool(s)
    except:
        return default_value

class ManualResult():
    def __init__(self, csvrow):
        self.project = csvrow[0]
        self.category = csvrow[1]
        self.sha = csvrow[2]
        self.baddataset = parse_bool(csvrow[3])
        self.newfiles = parse_bool(csvrow[4])
        self.repeatedchanges = parse_bool(csvrow[5])
        self.formatting = parse_bool(csvrow[6])
        self.tinypdg = parse_bool(csvrow[7])
        self.ok = parse_bool(csvrow[8])
        self.description = csvrow[9]
        
        if self.ok and any([self.baddataset, self.newfiles,
                            self.repeatedchanges, self.formatting,
                            self.tinypdg]):
            raise ValueError("Best checkout " + str([self.project, self.category, self.sha]))
        
    def __str__(self):
        return self.sha + " " + str(self.ok)
    
    def __repr__(self):
        return self.__str__()

def load_manual_analysis():
    """Opens the results from the manual analysis and loads them into something
    presentable."""
    PATH = "./manual-check-for-tangled-failure.csv"
    results = {}
    with open(PATH) as file:
        reader = csv.reader(file, delimiter=',')
        for row in reader:
            project = row[0]
            category = row[1]

            # Skip the header line
            if project == 'Project':
                continue
            
            if not project in results:
                results[project] = {}
            if not category in results[row[0]]:
                results[project][category] = []
            results[project][category].append(ManualResult(row))
    return results

def filter_results_with_manual(results, project_names, categories):
    """Creates a new results object with removal of commits we deemed not good
    enough in our manual analysis."""
    manual_results = load_manual_analysis()
    new_results = {}
    for project in project_names:
        new_results[project] = {}
        # First create the categories, since some commits might flip due to
        # being badly categorised. In this case we need the other category
        # to already exist.
        for category in categories:
            new_results[project][category] = []

        for category in categories:
            for result in results[project][category]:
                
                # Analyse if we want to keep it
                manual = list(filter(lambda r: r.sha == result.sha, manual_results[project][category]))
                if len(manual) != 1:
                    raise ValueError(manual)
                if manual[0].newfiles or manual[0].repeatedchanges or manual[0].formatting or manual[0].tinypdg:
                    continue
                
                # Flip it if need to
                if manual[0].baddataset:
                    other_category = 'atomic'
                    if category == 'atomic':
                        other_category = 'tangled'
                    new_results[project][other_category].append(result)
                    continue
                
                # Keep it
                new_results[project][category].append(result)
    return new_results


if __name__ == "__main__":
    #print(load_manual_analysis())
    results = load_manual_analysis()
    # Flatten it
    r = [res for project in results
         for category in results[project]
         for res in results[project][category]]
    newfiles = 0
    tinypdg = 0
    formatting = 0
    repeated = 0
    baddata = 0
    ok = 0
    for res in r:
        newfiles = newfiles + res.newfiles
        repeated = repeated + res.repeatedchanges
        formatting = formatting + res.formatting
        tinypdg = tinypdg + res.tinypdg
        if res.newfiles or res.repeatedchanges or res.formatting or res.tinypdg:
            continue
        elif res.baddataset:
            baddata = baddata + 1
        else:
            ok = ok + 1
    print('new', 'repeat', 'format', 'tinypdg', 'baddata', 'ok', sep='\t')
    print(newfiles, repeated, formatting, tinypdg, baddata, ok, sep='\t')